@@ -2,48 +2,26 @@
 
 #include "common.h"
 
-#include <ccnet.h>
-
 #define DEBUG_FLAG SEAFILE_DEBUG_SYNC
 #include "log.h"
 
 #include "seafile-error.h"
 #include "seafile-session.h"
-#include "index/index.h"
-#include "merge-recursive.h"
-#include "unpack-trees.h"
 #include "vc-utils.h"
 #include "utils.h"
 #include "seafile-config.h"
 
-#include "processors/checkff-proc.h"
+#include "timer.h"
 
 #define CLONE_DB "clone.db"
 
 #define CHECK_CONNECT_INTERVAL 5
 
-static void
-on_repo_fetched (SeafileSession *seaf,
-                 TransferTask *tx_task,
-                 SeafCloneManager *mgr);
-
 static void
 on_repo_http_fetched (SeafileSession *seaf,
                       HttpTxTask *tx_task,
                       SeafCloneManager *mgr);
 
-static void
-on_checkout_done (CheckoutTask *task, SeafRepo *repo, void *data);
-
-static int
-start_index_or_transfer (SeafCloneManager *mgr, CloneTask *task, GError **error);
-
-static void
-start_connect_task_relay (CloneTask *task, GError **error);
-
-static void
-start_checkout (SeafRepo *repo, CloneTask *task);
-
 static void
 transition_state (CloneTask *task, int new_state);
 
@@ -80,16 +58,6 @@ static const char *error_str[] = {
     "internal",
 };
 
-static void
-start_clone (CloneTask *task)
-{
-    SeafRepo *repo = seaf_repo_manager_get_repo (seaf->repo_mgr, task->repo_id);
-    if (!repo)
-        start_index_or_transfer (task->manager, task, NULL);
-    else if (repo->head == NULL)
-        start_checkout (repo, task);
-}
-
 static void
 mark_clone_done_v2 (SeafRepo *repo, CloneTask *task)
 {
@@ -177,7 +145,6 @@ start_clone_v2 (CloneTask *task)
         seaf_repo_manager_set_repo_email (seaf->repo_mgr, repo, task->email);
         seaf_repo_manager_set_repo_relay_info (seaf->repo_mgr, repo->id,
                                                task->peer_addr, task->peer_port);
-        seaf_repo_manager_set_repo_relay_id (seaf->repo_mgr, repo, task->peer_id);
         if (task->server_url) {
             seaf_repo_manager_set_repo_property (seaf->repo_mgr,
                                                  repo->id,
@@ -195,88 +162,6 @@ start_clone_v2 (CloneTask *task)
         transition_to_error (task, CLONE_ERROR_FETCH);
 }
 
-static void
-check_protocol_done_cb (CcnetProcessor *processor, gboolean success, void *data)
-{
-    CloneTask *task = data;
-
-    if (success)
-        task->server_side_merge = TRUE;
-    else if (processor->failure == PROC_NO_SERVICE)
-        /* Talking to an old server. */
-        task->server_side_merge = FALSE;
-
-    if (task->server_side_merge)
-        start_clone_v2 (task);
-    else
-        start_clone (task);
-}
-
-static int
-start_check_protocol_proc (const char *peer_id, CloneTask *task)
-{
-    CcnetProcessor *processor;
-
-    if (task->repo_version == 0) {
-        task->server_side_merge = FALSE;
-        start_clone (task);
-        return 0;
-    }
-
-    processor = ccnet_proc_factory_create_remote_master_processor (
-        seaf->session->proc_factory, "seafile-check-protocol", peer_id);
-    if (!processor) {
-        seaf_warning ("failed to create seafile-check-protocol proc.\n");
-        return -1;
-    }
-
-    if (ccnet_processor_startl (processor, NULL) < 0) {
-        seaf_warning ("failed to start seafile-check-protocol proc.\n");
-        return -1;
-    }
-
-    g_signal_connect (processor, "done", (GCallback)check_protocol_done_cb, task);
-
-    transition_state (task, CLONE_STATE_CHECK_PROTOCOL);
-
-    return 0;
-}
-
-static void
-start_connect_task_relay (CloneTask *task, GError **error)
-{
-    CcnetPeer *peer = ccnet_get_peer (seaf->ccnetrpc_client, task->peer_id);
-    if (!peer) {
-        /* clone from a new relay */
-        GString *buf = NULL; 
-        seaf_message ("add relay before clone, %s:%s\n",
-                      task->peer_addr, task->peer_port);
-        buf = g_string_new(NULL);
-        g_string_append_printf (buf, "add-relay --id %s --addr %s:%s",
-                                task->peer_id, task->peer_addr, task->peer_port);
-        ccnet_send_command (seaf->session, buf->str, NULL, NULL);
-        transition_state (task, CLONE_STATE_CONNECT);
-        g_string_free (buf, TRUE);
-    } else {
-        /* The peer is added to ccnet already and will be connected,
-         * only need to transition the state
-         */
-        transition_state (task, CLONE_STATE_CONNECT);
-    }
-
-    if (peer)
-        g_object_unref (peer);
-}
-
-static void
-connect_non_http_server (CloneTask *task)
-{
-    if (!ccnet_peer_is_ready (seaf->ccnetrpc_client, task->peer_id))
-        start_connect_task_relay (task, NULL);
-    else
-        start_check_protocol_proc (task->peer_id, task);
-}
-
 static void
 check_head_commit_done (HttpHeadCommit *result, void *user_data)
 {
@@ -353,7 +238,6 @@ check_http_fileserver_protocol_done (HttpProtocolVersion *result, void *user_dat
         task->http_protocol_version = result->version;
         task->effective_url = http_fileserver_url (task->server_url);
         task->use_fileserver_port = TRUE;
-        task->http_sync = TRUE;
         http_check_head_commit (task);
     } else {
         /* Wait for periodic retry. */
@@ -376,7 +260,6 @@ check_http_protocol_done (HttpProtocolVersion *result, void *user_data)
     if (result->check_success && !result->not_supported) {
         task->http_protocol_version = result->version;
         task->effective_url = g_strdup(task->server_url);
-        task->http_sync = TRUE;
         http_check_head_commit (task);
     } else if (strncmp(task->server_url, "https", 5) != 0) {
         char *host_fileserver = http_fileserver_url(task->server_url);
@@ -638,8 +521,6 @@ restart_task (sqlite3_stmt *stmt, void *data)
             transition_to_error (task, CLONE_ERROR_CHECK_SERVER);
             return TRUE;
         }
-    } else {
-        connect_non_http_server (task);
     }
 
     g_hash_table_insert (mgr->tasks, g_strdup(task->repo_id), task);
@@ -683,13 +564,6 @@ seaf_clone_manager_init (SeafCloneManager *mgr)
     return 0;
 }
 
-static void
-continue_task_when_peer_connected (CloneTask *task)
-{
-    if (ccnet_peer_is_ready (seaf->ccnetrpc_client, task->peer_id))
-        start_check_protocol_proc (task->peer_id, task);
-}
-
 static int check_connect_pulse (void *vmanager)
 {
     SeafCloneManager *mgr = vmanager;
@@ -705,8 +579,6 @@ static int check_connect_pulse (void *vmanager)
             task->err_detail = NULL;
             task->error = 0;
             check_http_protocol (task);
-        } else if (task->state == CLONE_STATE_CONNECT) {
-            continue_task_when_peer_connected (task);
         }
     }
 
@@ -716,19 +588,13 @@ static int check_connect_pulse (void *vmanager)
 int
 seaf_clone_manager_start (SeafCloneManager *mgr)
 {
-    ccnet_proc_factory_register_processor (seaf->session->proc_factory,
-                                           "seafile-checkff",
-                                           SEAFILE_TYPE_CHECKFF_PROC);
-
-    mgr->check_timer = ccnet_timer_new (check_connect_pulse, mgr,
-                                        CHECK_CONNECT_INTERVAL * 1000);
+    mgr->check_timer = seaf_timer_new (check_connect_pulse, mgr,
+                                       CHECK_CONNECT_INTERVAL * 1000);
 
     char *sql = "SELECT * FROM CloneTasks";
     if (sqlite_foreach_selected_row (mgr->db, sql, restart_task, mgr) < 0)
         return -1;
 
-    g_signal_connect (seaf, "repo-fetched",
-                      (GCallback)on_repo_fetched, mgr);
     g_signal_connect (seaf, "repo-http-fetched",
                       (GCallback)on_repo_http_fetched, mgr);
 
@@ -870,165 +736,26 @@ transition_to_error (CloneTask *task, int error)
 static int
 add_transfer_task (CloneTask *task, GError **error)
 {
-    if (!task->http_sync) {
-        task->tx_id = seaf_transfer_manager_add_download (seaf->transfer_mgr,
-                                                          task->repo_id,
-                                                          task->repo_version,
-                                                          task->peer_id,
-                                                          "fetch_head",
-                                                          "master",
-                                                          task->token,
-                                                          task->server_side_merge,
-                                                          task->passwd,
-                                                          task->worktree,
-                                                          task->email,
-                                                          error);
-        if (!task->tx_id)
-            return -1;
-    } else {
-        int ret = http_tx_manager_add_download (seaf->http_tx_mgr,
-                                                task->repo_id,
-                                                task->repo_version,
-                                                task->effective_url,
-                                                task->token,
-                                                task->server_head_id,
-                                                TRUE,
-                                                task->passwd,
-                                                task->worktree,
-                                                task->http_protocol_version,
-                                                task->email,
-                                                task->use_fileserver_port,
-                                                task->repo_name,
-                                                error);
-        if (ret < 0)
-            return -1;
-        task->tx_id = g_strdup(task->repo_id);
-    }
-
+    int ret = http_tx_manager_add_download (seaf->http_tx_mgr,
+                                            task->repo_id,
+                                            task->repo_version,
+                                            task->effective_url,
+                                            task->token,
+                                            task->server_head_id,
+                                            TRUE,
+                                            task->passwd,
+                                            task->worktree,
+                                            task->http_protocol_version,
+                                            task->email,
+                                            task->use_fileserver_port,
+                                            task->repo_name,
+                                            error);
+    if (ret < 0)
+        return -1;
+    task->tx_id = g_strdup(task->repo_id);
     return 0;
 }
 
-typedef struct {
-    CloneTask *task;
-    gboolean success;
-} IndexAux;
-
-static void *
-index_files_job (void *data)
-{
-    IndexAux *aux = data;
-    CloneTask *task = aux->task;
-
-    if (seaf_repo_index_worktree_files (task->repo_id, task->repo_version,
-                                        task->email,
-                                        task->worktree,
-                                        task->passwd, task->enc_version,
-                                        task->random_key,
-                                        task->root_id) == 0)
-        aux->success = TRUE;
-
-    return data;
-}
-
-static void
-index_files_done (void *result)
-{
-    IndexAux *aux = result;
-    CloneTask *task = aux->task;
-
-    if (!aux->success) {
-        transition_to_error (task, CLONE_ERROR_INDEX);
-        goto out;
-    }
-
-    if (task->state == CLONE_STATE_CANCEL_PENDING) {
-        transition_state (task, CLONE_STATE_CANCELED);
-        goto out;
-    }
-
-    if (add_transfer_task (task, NULL) < 0) {
-        transition_to_error (task, CLONE_ERROR_FETCH);
-        goto out;
-    }
-
-    transition_state (task, CLONE_STATE_FETCH);
-
-out:
-    g_free (aux);
-    return;
-}
-
-#ifndef WIN32
-
-static gboolean
-is_non_empty_directory (const char *path)
-{
-    GDir *dir;
-    GError *error = NULL;
-    gboolean ret = FALSE;
-
-    dir = g_dir_open (path, 0, &error);
-    if (dir != NULL && g_dir_read_name (dir) != NULL)
-        ret = TRUE;
-    if (dir)
-        g_dir_close (dir);
-
-    return ret;
-}
-
-#else
-
-static int
-check_empty_cb (wchar_t *parent, WIN32_FIND_DATAW *fdata, void *user_data, gboolean *stop)
-{
-    gboolean *res = user_data;
-
-    *res = TRUE;
-    *stop = TRUE;
-
-    return 0;
-}
-
-static gboolean
-is_non_empty_directory (const char *path)
-{
-    wchar_t *wpath = win32_long_path (path);
-    gboolean ret = FALSE;
-
-    traverse_directory_win32 (wpath, check_empty_cb, &ret);
-
-    return ret;
-}
-
-#endif  /* WIN32 */
-
-static int
-start_index_or_transfer (SeafCloneManager *mgr, CloneTask *task, GError **error)
-{
-    IndexAux *aux;
-    int ret = 0;
-
-    if (is_non_empty_directory (task->worktree)) {
-        transition_state (task, CLONE_STATE_INDEX);
-
-        aux = g_new0 (IndexAux, 1);
-        aux->task = task;
-
-        ccnet_job_manager_schedule_job (seaf->job_mgr,
-                                        index_files_job,
-                                        index_files_done,
-                                        aux);
-    } else {
-        ret = add_transfer_task (task, error);
-        if (ret == 0)
-            transition_state (task, CLONE_STATE_FETCH);
-        else
-            transition_to_error (task, CLONE_ERROR_FETCH);
-    }
-
-    return ret;
-}
-
 static gboolean
 is_duplicate_task (SeafCloneManager *mgr, const char *repo_id)
 {
@@ -1243,7 +970,7 @@ seaf_clone_manager_check_worktree_path (SeafCloneManager *mgr, const char *path,
     if (check_dir_inclusiveness (path, seaf->seaf_dir) != 0 ||
         /* It's OK if path is included by the default worktree parent. */
         check_dir_inclusiveness (path, seaf->worktree_dir) < 0 ||
-        check_dir_inclusiveness (path, seaf->session->config_dir) != 0) {
+        check_dir_inclusiveness (path, seaf->ccnet_dir) != 0) {
         seaf_warning ("Worktree path conflicts with seafile system path.\n");
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_GENERAL,
                      "Worktree conflicts system path");
@@ -1354,9 +1081,7 @@ add_task_common (SeafCloneManager *mgr,
             clone_task_free (task);
             return NULL;
         }
-    } else {
-        connect_non_http_server (task);
-    }
+    } 
 
     /* The old task for this repo will be freed. */
     g_hash_table_insert (mgr->tasks, g_strdup(task->repo_id), task);
@@ -1673,14 +1398,9 @@ seaf_clone_manager_cancel_task (SeafCloneManager *mgr,
     case CLONE_STATE_CHECK_SERVER:
         transition_state (task, CLONE_STATE_CANCEL_PENDING);
     case CLONE_STATE_FETCH:
-        if (!task->http_sync)
-            seaf_transfer_manager_cancel_task (seaf->transfer_mgr,
-                                               task->tx_id,
-                                               TASK_TYPE_DOWNLOAD);
-        else
-            http_tx_manager_cancel_task (seaf->http_tx_mgr,
-                                         task->repo_id,
-                                         HTTP_TASK_TYPE_DOWNLOAD);
+        http_tx_manager_cancel_task (seaf->http_tx_mgr,
+                                     task->repo_id,
+                                     HTTP_TASK_TYPE_DOWNLOAD);
         transition_state (task, CLONE_STATE_CANCEL_PENDING);
         break;
     case CLONE_STATE_INDEX:
@@ -1723,11 +1443,6 @@ seaf_clone_manager_remove_task (SeafCloneManager *mgr,
         return -1;
     }
 
-    if (task->tx_id)
-        seaf_transfer_manager_remove_task (seaf->transfer_mgr,
-                                           task->tx_id,
-                                           TASK_TYPE_DOWNLOAD);
-
     /* On-disk task should have been removed. */
 
     g_hash_table_remove (mgr->tasks, repo_id);
@@ -1748,628 +1463,9 @@ seaf_clone_manager_get_tasks (SeafCloneManager *mgr)
     return g_hash_table_get_values (mgr->tasks);
 }
 
-typedef struct {
-    gboolean is_fast_forward;
-    gboolean check_ff_in_thread;
-    CloneTask *task;
-    SeafRepo *repo;
-    gboolean success;
-} MergeAux;
-
-typedef struct {
-    gboolean fast_forward;
-    char root_id[41];
-} CompareAux;
-
-static gboolean
-compare_root (SeafCommit *commit, void *data, gboolean *stop)
-{
-    CompareAux *aux = data;
-
-    /* If we've found a match in another branch, stop traversing. */
-    if (aux->fast_forward) {
-        *stop = TRUE;
-        return TRUE;
-    }
-
-    if (strcmp (commit->root_id, aux->root_id) == 0) {
-        aux->fast_forward = TRUE;
-        *stop = TRUE;
-    }
-
-    return TRUE;
-}
-
-static gboolean
-check_fast_forward (SeafCommit *head, const char *root_id)
-{
-    CompareAux *aux = g_new0 (CompareAux, 1);
-    gboolean ret;
-
-    memcpy (aux->root_id, root_id, 41);
-    if (!seaf_commit_manager_traverse_commit_tree (seaf->commit_mgr,
-                                                   head->repo_id,
-                                                   head->version,
-                                                   head->commit_id,
-                                                   compare_root,
-                                                   aux, FALSE)) {
-        g_free (aux);
-        return FALSE;
-    }
-
-    ret = aux->fast_forward;
-    g_free (aux);
-    return ret;
-}
-
-#if 0
-static int 
-print_index (struct index_state *istate)
-{
-    int i;
-    struct cache_entry *ce;
-    char id[41];
-    g_message ("Totally %u entries in index, version %u.\n",
-               istate->cache_nr, istate->version);
-    for (i = 0; i < istate->cache_nr; ++i) {
-        ce = istate->cache[i];
-        rawdata_to_hex (ce->sha1, id, 20);
-        g_message ("%s, %s, %o, %"G_GUINT64_FORMAT", %s, %d\n",
-                   ce->name, id, ce->ce_mode, 
-                   ce->ce_mtime.sec, ce->modifier, ce_stage(ce));
-    }
-
-    return 0;
-}
-#endif
-
-static int
-real_merge (SeafRepo *repo, SeafCommit *head, CloneTask *task)
-{
-    struct merge_options opts;
-    char index_path[SEAF_PATH_MAX];
-    struct index_state istate;
-    char *root_id = NULL;
-    int clean;
-    int ret = 0;
-
-    memset (&istate, 0, sizeof(istate));
-    snprintf (index_path, SEAF_PATH_MAX, "%s/%s", repo->manager->index_dir, repo->id);
-    if (read_index_from (&istate, index_path, repo->version) < 0) {
-        seaf_warning ("Failed to load index.\n");
-        return -1;
-    }
-
-    init_merge_options (&opts);
-    memcpy (opts.repo_id, repo->id, 36);
-    opts.version = repo->version;
-    opts.index = &istate;
-    opts.worktree = task->worktree;
-    opts.ancestor = "common ancestor";
-    opts.branch1 = seaf->session->base.user_name;
-    opts.branch2 = head->creator_name;
-    opts.remote_head = head->commit_id;
-    /* Don't need to check locked files on windows. */
-    opts.force_merge = TRUE;
-    if (repo->encrypted) {
-        opts.crypt = seafile_crypt_new (repo->enc_version, 
-                                        repo->enc_key, 
-                                        repo->enc_iv);
-    }
-
-    /* Merge the downloaded branch with the current worktree contents.
-     * EMPTY_SHA1 represents an empty common ancestor tree.
-     */
-    merge_recursive (&opts,
-                     task->root_id, head->root_id, EMPTY_SHA1,
-                     &clean, &root_id);
-    g_free (root_id);
-
-    if (update_index (&istate, index_path) < 0) {
-        seaf_warning ("Failed to update index.\n");
-        ret = -1;
-    }
-
-    discard_index (&istate);
-    g_free (opts.crypt);
-    clear_merge_options (&opts);
-
-    return ret;
-}
-
-static int
-fast_forward_checkout (SeafRepo *repo, SeafCommit *head, CloneTask *task)
-{
-    SeafRepoManager *mgr = repo->manager;
-    char index_path[SEAF_PATH_MAX];
-    struct tree_desc trees[2];
-    struct unpack_trees_options topts;
-    struct index_state istate;
-    int ret = 0;
-
-    if (strcmp (head->root_id, task->root_id) == 0)
-        return 0;
-
-    memset (&istate, 0, sizeof(istate));
-    snprintf (index_path, SEAF_PATH_MAX, "%s/%s", mgr->index_dir, repo->id);
-    if (read_index_from (&istate, index_path, repo->version) < 0) {
-        seaf_warning ("Failed to load index.\n");
-        return -1;
-    }
-    repo->index_corrupted = FALSE;
-
-    fill_tree_descriptor (repo->id, repo->version, &trees[0], task->root_id);
-    fill_tree_descriptor (repo->id, repo->version, &trees[1], head->root_id);
-
-    memset(&topts, 0, sizeof(topts));
-    memcpy (topts.repo_id, repo->id, 36);
-    topts.version = repo->version;
-    topts.base = task->worktree;
-    topts.head_idx = -1;
-    topts.src_index = &istate;
-    topts.update = 1;
-    topts.merge = 1;
-    topts.fn = twoway_merge;
-    if (repo->encrypted) {
-        topts.crypt = seafile_crypt_new (repo->enc_version, 
-                                         repo->enc_key, 
-                                         repo->enc_iv);
-    }
-
-    if (unpack_trees (2, trees, &topts) < 0) {
-        seaf_warning ("Failed to merge commit %s with work tree.\n", head->commit_id);
-        ret = -1;
-        goto out;
-    }
-
-    if (update_worktree (&topts, FALSE,
-                         head->commit_id,
-                         head->creator_name,
-                         NULL) < 0) {
-        seaf_warning ("Failed to update worktree.\n");
-        /* Still finishe checkout even have I/O errors. */
-    }
-
-    discard_index (&istate);
-    istate = topts.result;
-
-    if (update_index (&istate, index_path) < 0) {
-        seaf_warning ("Failed to update index.\n");
-        ret = -1;
-    }
-
-out:
-    tree_desc_free (&trees[0]);
-    tree_desc_free (&trees[1]);
-
-    g_free (topts.crypt);
-
-    discard_index (&istate);
-
-    return ret;
-}
-
-static int
-create_index_branch (SeafRepo *repo, const char *root_id)
-{
-    SeafCommit *commit = NULL;
-    SeafBranch *branch = NULL;
-    int ret = 0;
-
-    commit = seaf_commit_new (NULL, repo->id, root_id,
-                              repo->email ? repo->email
-                              : seaf->session->base.user_name,
-                              seaf->session->base.id,
-                              "Temp commit for index", 0);
-    seaf_repo_to_commit (repo, commit);
-    if (seaf_commit_manager_add_commit (seaf->commit_mgr, commit) < 0) {
-        seaf_warning ("Failed to add commit.\n");
-        ret = -1;
-        goto out;
-    }
-
-    branch = seaf_branch_manager_get_branch (seaf->branch_mgr, repo->id, "index");
-    if (!branch) {
-        branch = seaf_branch_new ("index", repo->id, commit->commit_id);
-        if (seaf_branch_manager_add_branch (seaf->branch_mgr, branch) < 0) {
-            seaf_warning ("Failed to add branch.\n");
-            ret = -1;
-            goto out;
-        }
-    } else {
-        seaf_branch_set_commit (branch, commit->commit_id);
-        seaf_branch_manager_update_branch (seaf->branch_mgr, branch);
-    }
-
-out:
-    seaf_commit_unref (commit);
-    seaf_branch_unref (branch);
-    return ret;
-}
-
-static void *
-merge_job (void *data)
-{
-    MergeAux *aux = data;
-    CloneTask *task = aux->task;
-    SeafRepo *repo = aux->repo;
-    SeafBranch *local = NULL;
-    SeafCommit *head = NULL;
-    gboolean is_ff;
-
-    local = seaf_branch_manager_get_branch (seaf->branch_mgr, repo->id, "local");
-    if (!local) {
-        aux->success = FALSE;
-        goto out;
-    }
-
-    head = seaf_commit_manager_get_commit (seaf->commit_mgr,
-                                           repo->id, repo->version,
-                                           local->commit_id);
-    if (!head) {
-        aux->success = FALSE;
-        goto out;
-    }
-
-    if (aux->check_ff_in_thread)
-        is_ff = check_fast_forward (head, task->root_id);
-    else
-        is_ff = aux->is_fast_forward;
-
-    if (is_ff) {
-        seaf_debug ("[clone mgr] Fast forward.\n");
-        if (fast_forward_checkout (repo, head, task) < 0)
-            goto out;
-    } else {
-        if (real_merge (repo, head, task) < 0)
-            goto out;
-
-        /* Create a temp branch "index" which references to task->root_id,
-         * so that new changes from the worktree won't be removed by GC.
-         * This branch should be deleted on the first commit operation of
-         * the repo.
-         */
-        if (create_index_branch (repo, task->root_id) < 0)
-            goto out;
-    }
-
-    /* Save head id for GC. */
-    seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                         repo->id,
-                                         REPO_REMOTE_HEAD,
-                                         head->commit_id);
-    seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                         repo->id,
-                                         REPO_LOCAL_HEAD,
-                                         head->commit_id);
-
-    aux->success = TRUE;
-
-out:
-    seaf_branch_unref (local);
-    seaf_commit_unref (head);
-    return aux;
-}
-
-static void
-merge_job_done (void *data)
-{
-    MergeAux *aux = data;
-    CloneTask *task = aux->task;
-    SeafRepo *repo = aux->repo;
-    SeafBranch *local = NULL;
-
-    if (!aux->success) {
-        goto error;
-    }
-
-    seaf_repo_manager_set_repo_worktree (repo->manager,
-                                         repo,
-                                         task->worktree);
-
-    local = seaf_branch_manager_get_branch (seaf->branch_mgr, repo->id, "local");
-    if (!local) {
-        seaf_warning ("Cannot get branch local for repo %s(%.10s).\n",
-                      repo->name, repo->id);
-        goto error;
-    }
-    /* Set repo head to mark checkout done. */
-    seaf_repo_set_head (repo, local);
-    seaf_branch_unref (local);
-
-    if (repo->auto_sync) {
-        if (seaf_wt_monitor_watch_repo (seaf->wt_monitor, repo->id, repo->worktree) < 0) {
-            seaf_warning ("failed to watch repo %s(%.10s).\n", repo->name, repo->id);
-            goto error;
-        }
-    }
-
-    if (task->state == CLONE_STATE_CANCEL_PENDING)
-        transition_state (task, CLONE_STATE_CANCELED);
-    else if (task->state == CLONE_STATE_MERGE) {
-        transition_state (task, CLONE_STATE_DONE);
-    }
-
-    g_free (aux);
-    return;
-
-error:
-    g_free (aux);
-    transition_to_error (task, CLONE_ERROR_MERGE);
-    return;
-}
-
-static void
-setup_repo_without_checkout (SeafRepo *repo, SeafBranch *local, CloneTask *task)
-{
-    if (create_index_branch (repo, task->root_id) < 0) {
-        transition_to_error (task, CLONE_ERROR_MERGE);
-        return;
-    }
-
-    seaf_repo_manager_set_repo_worktree (repo->manager,
-                                         repo,
-                                         task->worktree);
-
-    /* Set repo head to mark checkout done. */
-    seaf_repo_set_head (repo, local);
-
-    if (repo->auto_sync) {
-        if (seaf_wt_monitor_watch_repo (seaf->wt_monitor, repo->id, repo->worktree) < 0) {
-            seaf_warning ("failed to watch repo %s(%.10s).\n", repo->name, repo->id);
-        }
-    }
-
-    /* Save head id for GC. */
-    seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                         repo->id,
-                                         REPO_REMOTE_HEAD,
-                                         local->commit_id);
-    seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                         repo->id,
-                                         REPO_LOCAL_HEAD,
-                                         local->commit_id);
-
-    transition_state (task, CLONE_STATE_DONE);
-}
-
-static void
-checkff_done (CcnetProcessor *processor, gboolean success, void *data)
-{
-    SeafileCheckffProc *proc = (SeafileCheckffProc *)processor;
-    CloneTask *task = data;
-
-    if (task->state == CLONE_STATE_CANCEL_PENDING) {
-        transition_state (task, CLONE_STATE_CANCELED);
-        return;
-    }
-
-    SeafRepo *repo = seaf_repo_manager_get_repo (seaf->repo_mgr, task->repo_id);
-    if (!repo) {
-        seaf_warning ("Failed to get repo %s.\n", task->repo_id);
-        transition_to_error (task, CLONE_ERROR_MERGE);
-        return;
-    }
-
-    MergeAux *aux = g_new0 (MergeAux, 1);
-    aux->task = task;
-    aux->repo = repo;
-
-    /* If checkff proc fails, we're talking to an older server which doesn't support
-     * this processor. In that case transfer manager should have downloaded
-     * all history commits from the server. So we can still check ff locally in the
-     * merge thread.
-     */
-    if (success) {
-        aux->is_fast_forward = proc->is_fast_forward;
-        aux->check_ff_in_thread = FALSE;
-    } else {
-        aux->is_fast_forward = FALSE;
-        aux->check_ff_in_thread = TRUE;
-    }
-
-    ccnet_job_manager_schedule_job (seaf->job_mgr,
-                                    merge_job,
-                                    merge_job_done,
-                                    aux);
-}
-
-static int
-start_checkff_proc (CloneTask *task)
-{
-    CcnetProcessor *processor;
-
-    processor = ccnet_proc_factory_create_remote_master_processor (
-        seaf->session->proc_factory, "seafile-checkff", task->peer_id);
-    if (!processor) {
-        seaf_warning ("failed to create checkff proc.\n");
-        return -1;
-    }
-
-    g_signal_connect (processor, "done", (GCallback)checkff_done, task);
-
-    if (ccnet_processor_startl (processor, task->repo_id, task->root_id, NULL) < 0) {
-        seaf_warning ("failed to start checkff proc.\n");
-        return -1;
-    }
-
-    return 0;
-}
-
-static void
-index_files_before_merge_done (void *result)
-{
-    IndexAux *aux = result;
-    CloneTask *task = aux->task;
-
-    if (!aux->success) {
-        transition_to_error (task, CLONE_ERROR_MERGE);
-        goto out;
-    }
-
-    if (task->state == CLONE_STATE_CANCEL_PENDING) {
-        transition_state (task, CLONE_STATE_CANCELED);
-        goto out;
-    }
-
-    if (start_checkff_proc (task) < 0) {
-        transition_to_error (task, CLONE_ERROR_MERGE);
-        goto out;
-    }
-
-out:
-    g_free (aux);
-}
-
-static void
-start_checkout (SeafRepo *repo, CloneTask *task)
-{
-    if (repo->encrypted && task->passwd != NULL) {
-        if (seaf_repo_manager_set_repo_passwd (seaf->repo_mgr,
-                                               repo,
-                                               task->passwd) < 0) {
-            seaf_warning ("[Clone mgr] failed to set passwd for %s.\n", repo->id);
-            transition_to_error (task, CLONE_ERROR_INTERNAL);
-            return;
-        }
-    } else if (repo->encrypted) {
-        seaf_warning ("[Clone mgr] Password is empty for encrypted repo %s.\n",
-                   repo->id);
-        transition_to_error (task, CLONE_ERROR_PASSWD);
-        return;
-    }
-
-    if (g_access (task->worktree, F_OK) != 0 &&
-        g_mkdir_with_parents (task->worktree, 0777) < 0) {
-        seaf_warning ("[clone mgr] Failed to create worktree %s.\n",
-                      task->worktree);
-        transition_to_error (task, CLONE_ERROR_CHECKOUT);
-        return;
-    }
-
-    SeafBranch *local;
-    SeafCommit *commit;
-
-    local = seaf_branch_manager_get_branch (seaf->branch_mgr, repo->id, "local");
-    if (!local) {
-        seaf_warning ("Cannot get branch local for repo %s(%.10s).\n",
-                      repo->name, repo->id);
-        transition_to_error (task, CLONE_ERROR_CHECKOUT);
-        return;
-    }
-
-    commit = seaf_commit_manager_get_commit (seaf->commit_mgr,
-                                             repo->id, repo->version,
-                                             local->commit_id);
-    if (!commit) {
-        seaf_warning ("Failed to get commit %s:%.8s.\n", repo->id, local->commit_id);
-        transition_to_error (task, CLONE_ERROR_CHECKOUT);
-        return;
-    }
-
-    /* If remote head's tree is empty, we don't need to checkout or merge.
-     * The result would be the current state of worktree anyway.
-     */
-    if (strcmp (commit->root_id, EMPTY_SHA1) == 0) {
-        setup_repo_without_checkout (repo, local, task);
-        seaf_branch_unref (local);
-        seaf_commit_unref (commit);
-        return;
-    }
-
-    seaf_branch_unref (local);
-    seaf_commit_unref (commit);
-
-    if (!is_non_empty_directory (task->worktree)) {
-        transition_state (task, CLONE_STATE_CHECKOUT);
-        seaf_repo_manager_add_checkout_task (seaf->repo_mgr,
-                                             repo,
-                                             task->worktree,
-                                             on_checkout_done,
-                                             task->manager);
-    } else {
-        transition_state (task, CLONE_STATE_MERGE);
-
-        if (task->root_id[0] == 0) {
-            /* If the task is restarted, root_id may not be calculated yet. */
-            IndexAux *aux = g_new0 (IndexAux, 1);
-            aux->task = task;
-
-            ccnet_job_manager_schedule_job (seaf->job_mgr,
-                                            index_files_job,
-                                            index_files_before_merge_done,
-                                            aux);
-        } else {
-            /* Since we don't have complete history on the client, start a 
-             * processor to check if task->root_id exist in the history on the server.
-             * That means there is no change in the worktree after the last unsync.
-             */
-            if (start_checkff_proc (task) < 0) {
-                transition_to_error (task, CLONE_ERROR_MERGE);
-                return;
-            }
-        }
-    }
-}
-
 static void
 check_folder_permissions (CloneTask *task);
 
-static void
-on_repo_fetched (SeafileSession *seaf,
-                 TransferTask *tx_task,
-                 SeafCloneManager *mgr)
-{
-    CloneTask *task;
-
-    /* Only handle clone task. */
-    if (!tx_task->is_clone)
-        return;
-
-    task = g_hash_table_lookup (mgr->tasks, tx_task->repo_id);
-    g_return_if_fail (task != NULL);
-
-    if (tx_task->state == TASK_STATE_CANCELED) {
-        /* g_assert (task->state == CLONE_STATE_CANCEL_PENDING); */
-        transition_state (task, CLONE_STATE_CANCELED);
-        return;
-    } else if (tx_task->state == TASK_STATE_ERROR) {
-        /* transition_to_error (task, CLONE_ERROR_FETCH); */
-        /* Always restart failed clone task. */
-        if (add_transfer_task (task, NULL) == 0)
-            transition_state (task, CLONE_STATE_FETCH);
-        else
-            transition_to_error (task, CLONE_ERROR_FETCH);
-        return;
-    }
-
-    SeafRepo *repo = seaf_repo_manager_get_repo (seaf->repo_mgr,
-                                                 tx_task->repo_id);
-    if (repo == NULL) {
-        seaf_warning ("[Clone mgr] cannot find repo %s after fetched.\n", 
-                   tx_task->repo_id);
-        transition_to_error (task, CLONE_ERROR_INTERNAL);
-        return;
-    }
-
-    seaf_repo_manager_set_repo_token (seaf->repo_mgr, repo, task->token);
-    seaf_repo_manager_set_repo_email (seaf->repo_mgr, repo, task->email);
-    seaf_repo_manager_set_repo_relay_info (seaf->repo_mgr, repo->id,
-                                           task->peer_addr, task->peer_port);
-    seaf_repo_manager_set_repo_relay_id (seaf->repo_mgr, repo, task->peer_id);
-    if (task->server_url) {
-        seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                             repo->id,
-                                             REPO_PROP_SERVER_URL,
-                                             task->server_url);
-    }
-
-    if (!task->server_side_merge)
-        start_checkout (repo, task);
-    else
-        check_folder_permissions (task);
-}
-
 static void
 on_repo_http_fetched (SeafileSession *seaf,
                       HttpTxTask *tx_task,
@@ -2407,7 +1503,6 @@ on_repo_http_fetched (SeafileSession *seaf,
     seaf_repo_manager_set_repo_email (seaf->repo_mgr, repo, task->email);
     seaf_repo_manager_set_repo_relay_info (seaf->repo_mgr, repo->id,
                                            task->peer_addr, task->peer_port);
-    seaf_repo_manager_set_repo_relay_id (seaf->repo_mgr, repo, task->peer_id);
     if (task->server_url) {
         seaf_repo_manager_set_repo_property (seaf->repo_mgr,
                                              repo->id,
@@ -2418,34 +1513,6 @@ on_repo_http_fetched (SeafileSession *seaf,
     check_folder_permissions (task);
 }
 
-static void
-on_checkout_done (CheckoutTask *ctask, SeafRepo *repo, void *data)
-{
-    SeafCloneManager *mgr = data;
-    CloneTask *task = g_hash_table_lookup (mgr->tasks, repo->id);
-    g_return_if_fail (task != NULL);
-
-    if (!ctask->success) {
-        transition_to_error (task, CLONE_ERROR_CHECKOUT);
-        return;
-    }
-
-    if (task->state == CLONE_STATE_CANCEL_PENDING)
-        transition_state (task, CLONE_STATE_CANCELED);
-    else if (task->state == CLONE_STATE_CHECKOUT) {
-        /* Save repo head if for GC. */
-        seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                             repo->id,
-                                             REPO_REMOTE_HEAD,
-                                             repo->head->commit_id);
-        seaf_repo_manager_set_repo_property (seaf->repo_mgr,
-                                             repo->id,
-                                             REPO_LOCAL_HEAD,
-                                             repo->head->commit_id);
-        transition_state (task, CLONE_STATE_DONE);
-    }
-}
-
 static void
 check_folder_perms_done (HttpFolderPerms *result, void *user_data)
 {
