@@ -141,55 +141,12 @@ seaf_repo_set_name (SeafRepo *repo, const char *new_name);
 GList *
 seaf_repo_get_commits (SeafRepo *repo);
 
-int
-seaf_repo_index_add (SeafRepo *repo, const char *path);
-
-int
-seaf_repo_index_worktree_files (const char *repo_id,
-                                int version,
-                                const char *modifier,
-                                const char *worktree,
-                                const char *passwd,
-                                int enc_version,
-                                const char *random_key,
-                                char *root_id);
-
-int
-seaf_repo_index_rm (SeafRepo *repo, const char *path);
-
-char *
-seaf_repo_status (SeafRepo *repo);
-
-gboolean
-seaf_repo_is_worktree_changed (SeafRepo *repo);
-
-gboolean
-seaf_repo_is_index_unmerged (SeafRepo *repo);
-
 char *
 seaf_repo_index_commit (SeafRepo *repo, const char *desc,
                         gboolean is_force_commit,
                         gboolean is_initial_commit,
                         GError **error);
 
-int
-seaf_repo_checkout (SeafRepo *repo, const char *worktree_parent, char **error);
-
-int
-seaf_repo_checkout_commit (SeafRepo *repo, SeafCommit *commit, gboolean recover_merge,
-                           char **error);
-
-enum {
-    MERGE_STATUS_UNKNOWN = 0,
-    MERGE_STATUS_UPTODATE,
-    MERGE_STATUS_FAST_FORWARD,
-    MERGE_STATUS_REAL_MERGE,
-};
-
-int
-seaf_repo_merge (SeafRepo *repo, const char *branch, char **error,
-                 int *merge_status);
-
 GList *
 seaf_repo_diff (SeafRepo *repo, const char *old, const char *new, int fold_dir_diff, char **error);
 
@@ -321,68 +278,6 @@ seaf_repo_manager_set_repo_passwd (SeafRepoManager *manager,
                                    SeafRepo *repo,
                                    const char *passwd);
 
-int
-seaf_repo_manager_set_repo_relay_id (SeafRepoManager *mgr,
-                                     SeafRepo *repo,
-                                     const char *relay_id);
-
-int
-seaf_repo_manager_set_merge (SeafRepoManager *manager,
-                             const char *repo_id,
-                             const char *remote_head);
-
-int
-seaf_repo_manager_clear_merge (SeafRepoManager *manager,
-                               const char *repo_id);
-
-typedef struct {
-    gboolean in_merge;
-    char remote_head[41];
-} SeafRepoMergeInfo;
-
-int
-seaf_repo_manager_get_merge_info (SeafRepoManager *manager,
-                                  const char *repo_id,
-                                  SeafRepoMergeInfo *info);
-
-int
-seaf_repo_manager_get_common_ancestor (SeafRepoManager *manager,
-                                       const char *repo_id,
-                                       char *common_ancestor,
-                                       char *head_id);
-
-int
-seaf_repo_manager_set_common_ancestor (SeafRepoManager *manager,
-                                       const char *repo_id,
-                                       const char *common_ancestor,
-                                       const char *head_id);
-
-typedef struct {
-    char repo_id[41];
-    char worktree[SEAF_PATH_MAX];
-    int total_files;
-    int finished_files;
-    gboolean success;
-} CheckoutTask;
-
-typedef void (*CheckoutDoneCallback) (CheckoutTask *, SeafRepo *, void *);
-
-int
-seaf_repo_manager_add_checkout_task (SeafRepoManager *mgr,
-                                     SeafRepo *repo,
-                                     const char *worktree,
-                                     CheckoutDoneCallback done_cb,
-                                     void *cb_data);
-
-CheckoutTask *
-seaf_repo_manager_get_checkout_task (SeafRepoManager *mgr,
-                                     const char *repo_id);
-int
-seaf_repo_manager_update_repo_relay_info (SeafRepoManager *mgr,
-                                          SeafRepo *repo,
-                                          const char *new_addr,
-                                          const char *new_port);
-
 int
 seaf_repo_manager_update_repos_server_host (SeafRepoManager *mgr,
                                             const char *old_host,
@@ -427,9 +322,7 @@ struct _TransferTask;
 struct _HttpTxTask;
 
 int
-seaf_repo_fetch_and_checkout (struct _TransferTask *task,
-                              struct _HttpTxTask *http_task,
-                              gboolean is_http,
+seaf_repo_fetch_and_checkout (struct _HttpTxTask *http_task,
                               const char *remote_head_id);
 
 gboolean
